local spec = {}
local icons = {}
local classes = {}
local f = CreateFrame("Frame")
local frames_to_update = {}
local in_instance

for classID=1,MAX_CLASSES do
	local _,classTag = GetClassInfoByID(classID)
	local numTabs = GetNumSpecializationsForClassID(classID)
	classes[classTag] = {}
	for i=1,numTabs do
		local _,spec_name,_,spec_icon = GetSpecializationInfoForClassID(classID,i)
		classes[classTag][spec_name] = spec_icon
	end
end

local function fix_alpha(frame)
	local inRange,checkedRange = UnitInRange(frame.displayedUnit)
	if checkedRange and not inRange then
		if in_instance then
			frame:SetAlpha(0.25)
		else
			frame:SetAlpha(1)
		end
	else
		frame:SetAlpha(1)
	end
end

local function fix_right_click(frame)
	frame:RegisterForClicks("AnyDown","AnyUp")
	frame:SetAttribute("type2","macro")
	frame:SetAttribute("macrotext2","/run MouselookStart()")
end

hooksecurefunc("CompactUnitFrame_UpdateInRange",fix_alpha)

hooksecurefunc("CompactUnitFrame_UpdateName",function(frame)
	frame.name:SetText(GetUnitName(frame.unit,true):match("[^-]+"))
	local _,class = UnitClass(frame.unit)
	local color = RAID_CLASS_COLORS[class]
	if color then
		frame.name:SetTextColor(color.r,color.g,color.b)
		f:Show()
		if not frames_to_update[frame] then
			frames_to_update[frame] = true
			frame:SetScript("OnUpdate",nil)
		end
		frame.name:Show()
	end
	if not InCombatLockdown() then
		fix_right_click(frame)
		for i=1,3 do
			fix_right_click(_G[frame:GetName().."Buff"..i])
			fix_right_click(_G[frame:GetName().."Debuff"..i])
		end
		fix_right_click(_G[frame:GetName().."CenterStatusIcon"])
	end
end)

f:SetScript("OnEvent",function(self,event,...)
	if event=="PLAYER_ENTERING_WORLD" then
		local _,_,_,_,_,_,_,instanceMapID = GetInstanceInfo()
		in_instance = IsInInstance() or instanceMapID==1191
		if in_instance then
			f:RegisterEvent("PLAYER_SPECIALIZATION_CHANGED")
		else
			f:UnregisterEvent("PLAYER_SPECIALIZATION_CHANGED")
		end
	elseif in_instance then
		f:Show()
	end
	if event=="GROUP_ROSTER_UPDATE" and not InCombatLockdown() then
		if not IsInGroup() then
			icons = {}
			spec = {}
		end
		for k,v in pairs(frames_to_update) do
			if not k:IsShown() then
				frames_to_update[k] = nil
			end
		end
	end
end)
f:RegisterEvent("PLAYER_ENTERING_WORLD")
f:RegisterEvent("GROUP_ROSTER_UPDATE")

f:SetScript("OnUpdate",function(self,elapsed)
	self.timer = self.timer and self.timer+elapsed or 0
	if self.timer>0.5 then
		self.timer = 0
		RequestBattlefieldScoreData()
		for index=1,GetNumBattlefieldScores() do
			local name,_,_,_,_,faction,_,_,classToken,_,_,_,_,_,_,talentSpec = GetBattlefieldScore(index)
			if classToken then
				icons[name] = classes[classToken][talentSpec]
				spec[name] = talentSpec
			end
		end
		for k,v in pairs(frames_to_update) do
			k:SetScript("OnUpdate",nil)
			if not k:IsShown() or not k.unit then
				frames_to_update[k] = nil
			else
				fix_alpha(k)
				local texture = icons[GetUnitName(k.unit,true)]
				if texture then
					k.roleIcon:SetTexture(texture)
					k.roleIcon:SetTexCoord(0,1,0,1)
					k.roleIcon:SetSize(16,16)
					k.roleIcon:Show()
				end
			end
		end
	end
end)

hooksecurefunc(GameTooltip,"SetUnit",function(self,unit)
	if GetMouseFocus():GetName():match("^CompactRaidFrame%d") then
		local spec_name = spec[GetUnitName(unit,true)]
		if spec_name then
			GameTooltip:AddLine(spec_name,1,1,1)
			GameTooltip:Show()
		end
	end
end)